dofile_once("mods/weather/files/misc/global_multiplier.lua")

setting_cache = setting_cache or {}
local function get_setting(name)
    setting_cache[name] = setting_cache[name] or ModSettingGet(name)
    return setting_cache[name]
end

local entity_id = GetUpdatedEntityID()
local initialized = entity_id > -1
if entity_id < 0 then
	entity_id = EntityCreateNew()
end
local _, _, day, hour, minute, second = GameGetDateAndTimeLocal()
SetRandomSeed(day + hour + second, minute + second + entity_id)

local delay = Random(0, 60*60*4) * global_prob -- normal delay time
if not initialized and Random(100) < 75 then
  delay = Random(0, 60*10) * global_prob -- shorter delay
end
local script_source_file = "data/scripts/buildings/random_rain_spawner.lua"

local rain_prob = get_setting("weather.rain_probability")
rain_prob = (100 - rain_prob)

local rain_types_raw =
{
	{
		setting = "weather.enable_random_rain",
		prob = rain_prob,
		material = "",
		num_particles = { min = 0, max = 0 },
		duration = { min = 60*60*0.5, max = 60*60*4 },
		particles_bounce = true,
	},
	{
		setting = "weather.rain_water",
		prob = 4,
		material = "water",
		num_particles = { min = 10, max = 15 },
		duration = { min = 60*60*0.5, max = 60*60*4 },
		particles_bounce = true,
	},
	{
		setting = "weather.rain_radioactive_liquid",
		prob = 0.5,
		material = "radioactive_liquid",
		num_particles = { min = 10, max = 15 },
		duration = { min = 60*60*0.5, max = 60*60*4 },
		particles_bounce = true,
	},
	{
		setting = "weather.rain_acid",
		prob = 0.75,
		material = "acid",
		num_particles = { min = 10, max = 15 },
		duration = { min = 60*60*0.5, max = 60*60*4 },
		particles_bounce = true,
	},
	{
		setting = "weather.rain_blood",
		prob = 0.1,
		material = "blood",
		num_particles = { min = 10, max = 15 },
		duration = { min = 60*60*0.5, max = 60*60*4 },
		particles_bounce = true,
	},
	{
		setting = "weather.rain_gunpowder_unstable",
		prob = 0.1,
		material = "gunpowder_unstable",
		num_particles = { min = 1, max = 2 },
		duration = { min = 60*60*0.5, max = 60*60*1 },
		particles_bounce = true,
	},
	{
		setting = "weather.rain_lava",
		prob = 0.3,
		material = "lava",
		num_particles = { min = 10, max = 15 },
		duration = { min = 60*60*0.5, max = 60*60*4 },
		particles_bounce = false,
	},
	{
		setting = "weather.rain_slime",
		prob = 0.25,
		material = "slime",
		num_particles = { min = 1, max = 4 },
		duration = { min = 60*60*0.5, max = 60*60*4 },
		particles_bounce = true,
    },
	{
		setting = "weather.rain_gold",
		prob = 0.025,
		material = "gold",
		num_particles = { min = 1, max = 4 },
		duration = { min = 60*60*0.5, max = 60*60*2 },
		particles_bounce = true,
	}
}

g_rain_types = { total_prob = 0 }

for _, rain_type in ipairs(rain_types_raw) do
	if get_setting(rain_type.setting) then
		table.insert(g_rain_types, rain_type)
		g_rain_types.total_prob = g_rain_types.total_prob + rain_type.prob
	end
end

function init_total_prob(value)
	value.total_prob = 0
	for i,v in ipairs(value) do
		if v.prob ~= nil then
			value.total_prob = value.total_prob + v.prob
		end
	end
end

function random_from_table(what)
	if what.total_prob == 0 then
		init_total_prob(what)
	end

	local r = Randomf() * what.total_prob
	for i,v in ipairs(what) do
		if v.prob ~= nil then
			if r <= v.prob and (v.spawn_check == nil or v.spawn_check()) then
				return v
			end
			r = r - v.prob
		end
	end

	return nil
end

if initialized then
	local players = EntityGetWithTag("player_unit")
	if #players > 0 then
		local rain_entity = EntityCreateNew()
		local rain_type = random_from_table(g_rain_types)
		EntityAddComponent(rain_entity, "InheritTransformComponent")

		local lifetime = Random(rain_type.duration.min, rain_type.duration.max)
		delay = delay + lifetime

		EntityAddComponent(rain_entity, "LifetimeComponent", {
			lifetime=lifetime
		})
    EntityAddComponent(rain_entity, "LuaComponent", {
      script_source_file="data/scripts/buildings/random_rain.lua",
      execute_every_n_frame=1,
		})
		EntityAddComponent(rain_entity, "VariableStorageComponent", {
			name="material",
			value_string=rain_type.material
		})
		EntityAddComponent(rain_entity, "VariableStorageComponent", {
			name="num_particles",
			value_int=Random(rain_type.num_particles.min, rain_type.num_particles.max),
		})
		EntityAddComponent(rain_entity, "VariableStorageComponent", {
			name="particles_bounce",
			value_bool=rain_type.particles_bounce and "1" or "0"
		})
	end
end

EntityAddComponent(entity_id, "LuaComponent", {
	script_source_file=script_source_file,
	execute_on_added="0",
	remove_after_executed="1",
	execute_every_n_frame=delay,
})